import { h, useRef } from '@/h';
import { ResponseText, ResponseMarkdown, Markdown } from '@/modules/ResponseText';
import { aiChatStore, store } from '@/store/ai-chat-store';
import clsx from 'clsx';
export type MessageData = {
  content: string;
  role: 'user' | 'assistant' | 'system';
  response?: Response | null;
  className?: string;
  onFinish?: (text: string) => void;
  // onFinish之前，每200ms调用一次
  onChange?: (text: string) => void;
  id?: string;
};
type MessageProps = {} & MessageData;
const UserMessage = (props: MessageProps) => {
  return (
    <div className={`whitespace-pre message-item ${props.role}`} id={props.id}>
      {props.content}
    </div>
  );
};
export const AssistantMessage = (props: MessageProps) => {
  if (props.response) {
    return <ResponseMarkdown className={clsx(props.className)} onChange={props.onChange} response={props.response} onFinish={props.onFinish} />;
  }
  return <Markdown className={clsx(props.className)} markdown={props.content} />;
};

export const Message = (props: MessageProps) => {
  const isUser = props.role === 'user';
  const WrapperMessage = isUser ? UserMessage : AssistantMessage;
  return (
    <div className={clsx('pt-6 w-[80%] mx-auto chat-message-wrapper', props.className)}>
      <div className={clsx('chat-message')}>
        <div
          className={clsx(
            {
              'message-user': isUser,
              'message-assistant': !isUser,
            },
            'flex flex-col',
          )}>
          <div className='message-content-wrapper'>
            <div className={clsx('message-avatar flex gap-3 items-center', isUser && 'hidden')}>
              <img src='https://minio.xiongxiao.me/resources/root/avatar.png' alt='' className='w-6 h-6 rounded-full' />
              <div>AI 助手</div>
            </div>
            <div className='message-content scrollbar overflow-x-hidden'>
              <WrapperMessage {...props} />
            </div>
          </div>
        </div>
      </div>
    </div>
  );
};
export const AiChat = () => {
  const ref = useRef();
  const aiMessageRef = useRef();
  const textareaRef = useRef<HTMLTextAreaElement>();
  const scrollContainer = useRef();
  store.subscribe(
    (state, prevState) => {
      const messages = state.messages;
      // console.log('store', state, prevState);
      if (aiMessageRef.current) {
        messages.forEach((message) => {
          const id = message.id;
          if (document.getElementById('msg-' + id)) {
            return;
          }

          // let msg: JSX.Element;
          // if (message.role === 'ai' && message.response) {
          //   msg = ResponseMarkdown({ response: message.response, className: 'message-item ai', onFinish: () => {} });
          // } else if (message.role === 'ai' && message.content) {
          //   msg = AssistantMessage({ ...message, className: 'message-item ai' });
          // } else {
          //   msg = UserMessage({ ...message, id });
          // }
          const onChange = (text: string) => {
            // const scrollBottom = container.scrollHeight - container.scrollTop - container.clientHeight;
            const scrollContainerEl = scrollContainer.current;
            const bottom = document.querySelector('.ai-message-bottom');
            if (scrollContainerEl) {
              // 如果滚动条距离底部小于100px，自动滚动到底部
              if (scrollContainerEl.scrollHeight - scrollContainerEl.scrollTop - scrollContainerEl.clientHeight < 400) {
              }
              // console.log('scrollContainerEl', scrollContainerEl.scrollHeight, scrollContainerEl.scrollTop, scrollContainerEl.clientHeight);
              // console.log('scrollContainerEl<400', scrollContainerEl.scrollHeight - scrollContainerEl.scrollTop - scrollContainerEl.clientHeight < 400);
              bottom?.scrollIntoView({ behavior: 'smooth' });
            }
          };
          const onFinish = (text: string) => {
            aiChatStore.getState().setEnd(true);
          };
          const msg = Message({ ...message, id, onChange, onFinish });
          aiMessageRef.current!.appendChild(msg);
        });
      }
    },
    { key: 'aiStore', path: 'messages' },
  );
  const onSend = () => {
    const input = textareaRef.current;
    const state = aiChatStore.getState();
    if (state.loading || !state.end) {
      // console.log('loading');
      return;
    }
    if (input && input.value.trim() !== '') {
      aiChatStore.getState().sendMessage(input.value);
      input.value = '';
    } else {
      console.log('输入为空');
    }
  };
  return (
    <div
      ref={ref}
      className='w-full h-full  bg-gray-100 flex justify-center'
      onLoad={(e) => {
        // console.log('onLoad======', e, ref.current);
      }}>
      <div className='w-[80%] border border-gray-200 p-2 shadow bg-white mt-2 mb-2 rounded-lg  flex flex-col'>
        <h1 className='text-2xl font-bold text-gray-800 text-center'>kevisual助手</h1>
        <div className='relative flex-grow overflow-auto scrollbar' ref={scrollContainer}>
          <div className='message mt-2 mb-5 m-h-[40px]' id='ai-message-content' ref={aiMessageRef}></div>
          <div className='ai-message-bottom mb-4'></div>
          <div className='flex w-full sticky justify-center bottom-0'>
            <div className='w-[80%] flex flex-col gap-2 '>
              <label htmlFor='inputField' className='text-lg  font-medium text-gray-700'>
                输入框
              </label>
              <textarea
                id='inputField'
                ref={textareaRef}
                onInput={(e) => {
                  // console.log(e.target?.value); // 控制台输出输入的值
                }}
                onKeyDown={(e) => {
                  if (e.key === 'Enter' && !e.shiftKey) {
                    e.preventDefault(); // 防止换行
                    onSend();
                  }
                }}
                className='w-full px-4 py-2 border border-gray-200 rounded-md focus:outline-none focus:ring-2 focus:ring-blue-500 focus:border-blue-500 transition duration-200'
                placeholder='请输入内容'
              />
              <button
                className='bg-blue-500 text-white p-2 rounded disabled:bg-gray-400'
                onClick={() => {
                  onSend();
                }}>
                提交
              </button>
            </div>
          </div>
        </div>
      </div>
    </div>
  );
};
